@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -175,14 +175,10 @@ filter_reset (GMimeFilter *filter)
  * Creates a new #GMimeFilterStrip filter which will strip trailing
  * whitespace from every line of input passed through the filter.
  *
- * Returns a new strip filter.
+ * Returns: a new strip filter.
  **/
 GMimeFilter *
 g_mime_filter_strip_new (void)
 {
-	GMimeFilterStrip *new;
-	
-	new = g_object_new (GMIME_TYPE_FILTER_STRIP, NULL);
-	
-	return (GMimeFilter *) new;
+	return g_object_newv (GMIME_TYPE_FILTER_STRIP, 0, NULL);
 }
