@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -49,6 +49,7 @@ struct _GMimeGpgContext {
 	GMimeCryptoContext parent_object;
 	gboolean auto_key_retrieve;
 	gboolean always_trust;
+	gboolean use_agent;
 	char *path;
 };
 
@@ -69,6 +70,9 @@ void g_mime_gpg_context_set_auto_key_retrieve (GMimeGpgContext *ctx, gboolean au
 gboolean g_mime_gpg_context_get_always_trust (GMimeGpgContext *ctx);
 void g_mime_gpg_context_set_always_trust (GMimeGpgContext *ctx, gboolean always_trust);
 
+gboolean g_mime_gpg_context_get_use_agent (GMimeGpgContext *ctx);
+void g_mime_gpg_context_set_use_agent (GMimeGpgContext *ctx, gboolean use_agent);
+
 G_END_DECLS
 
 #endif /* __GMIME_GPG_CONTEXT_H__ */
