@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -55,7 +55,7 @@ typedef struct _GMimeMessageClass GMimeMessageClass;
 typedef enum _GMimeRecipientType {
 	GMIME_RECIPIENT_TYPE_TO,
 	GMIME_RECIPIENT_TYPE_CC,
-	GMIME_RECIPIENT_TYPE_BCC,
+	GMIME_RECIPIENT_TYPE_BCC
 } GMimeRecipientType;
 
 
@@ -126,6 +126,8 @@ void g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part
 void g_mime_message_foreach (GMimeMessage *message, GMimeObjectForeachFunc callback,
 			     gpointer user_data);
 
+GMimeObject *g_mime_message_get_body (GMimeMessage *message);
+
 G_END_DECLS
 
 #endif /* __GMIME_MESSAGE_H__ */
