@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2011 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -28,7 +28,7 @@
 
 #include "gmime-multipart-encrypted.h"
 #include "gmime-stream-filter.h"
-#include "gmime-filter-crlf.h"
+#include "gmime-filter-basic.h"
 #include "gmime-filter-from.h"
 #include "gmime-filter-crlf.h"
 #include "gmime-stream-mem.h"
@@ -144,7 +144,7 @@ g_mime_multipart_encrypted_new (void)
  * @ctx: encryption context
  * @sign: %TRUE if the content should also be signed or %FALSE otherwise
  * @userid: user id to use for signing (only used if @sign is %TRUE)
- * @hash: digest algorithm to use when signing
+ * @digest: digest algorithm to use when signing
  * @recipients: an array of recipients to encrypt to
  * @err: a #GError
  *
@@ -160,7 +160,7 @@ g_mime_multipart_encrypted_new (void)
 int
 g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *content,
 				    GMimeCryptoContext *ctx, gboolean sign,
-				    const char *userid, GMimeCryptoHash hash,
+				    const char *userid, GMimeDigestAlgo digest,
 				    GPtrArray *recipients, GError **err)
 {
 	GMimeStream *filtered_stream, *ciphertext, *stream;
@@ -196,7 +196,7 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	
 	/* encrypt the content stream */
 	ciphertext = g_mime_stream_mem_new ();
-	if (g_mime_crypto_context_encrypt (ctx, sign, userid, hash, recipients, stream, ciphertext, err) == -1) {
+	if (g_mime_crypto_context_encrypt (ctx, sign, userid, digest, recipients, stream, ciphertext, err) == -1) {
 		g_object_unref (ciphertext);
 		g_object_unref (stream);
 		return -1;
@@ -242,6 +242,33 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 }
 
 
+static GMimeStream *
+g_mime_data_wrapper_get_decoded_stream (GMimeDataWrapper *wrapper)
+{
+	GMimeStream *decoded_stream;
+	GMimeFilter *decoder;
+	
+	g_mime_stream_reset (wrapper->stream);
+	
+	switch (wrapper->encoding) {
+	case GMIME_CONTENT_ENCODING_BASE64:
+	case GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE:
+	case GMIME_CONTENT_ENCODING_UUENCODE:
+		decoder = g_mime_filter_basic_new (wrapper->encoding, FALSE);
+		decoded_stream = g_mime_stream_filter_new (wrapper->stream);
+		g_mime_stream_filter_add (GMIME_STREAM_FILTER (decoded_stream), decoder);
+		g_object_unref (decoder);
+		break;
+	default:
+		decoded_stream = wrapper->stream;
+		g_object_ref (wrapper->stream);
+		break;
+	}
+	
+	return decoded_stream;
+}
+
+
 /**
  * g_mime_multipart_encrypted_decrypt:
  * @mpe: multipart/encrypted object
@@ -252,8 +279,8 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
  * Attempts to decrypt the encrypted MIME part contained within the
  * multipart/encrypted object @mpe using the @ctx decryption context.
  *
- * If @result is non-NULL, then on a successful decrypt operation, it will be
- * updated to point to a newly-allocated #GMimeDecryptionResult with signature
+ * If @result is non-%NULL, then on a successful decrypt operation, it will be
+ * updated to point to a newly-allocated #GMimeDecryptResult with signature
  * status information as well as a list of recipients that the part was
  * encrypted to.
  *
@@ -263,16 +290,16 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
  **/
 GMimeObject *
 g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCryptoContext *ctx,
-				    GMimeDecryptionResult **result, GError **err)
+				    GMimeDecryptResult **result, GError **err)
 {
 	GMimeObject *decrypted, *version, *encrypted;
 	GMimeStream *stream, *ciphertext;
 	const char *protocol, *supported;
 	GMimeStream *filtered_stream;
 	GMimeContentType *mime_type;
-	GMimeDecryptionResult *res;
 	GMimeDataWrapper *wrapper;
 	GMimeFilter *crlf_filter;
+	GMimeDecryptResult *res;
 	GMimeParser *parser;
 	char *content_type;
 	
@@ -330,7 +357,7 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCryptoCon
 	
 	/* get the ciphertext stream */
 	wrapper = g_mime_part_get_content_object (GMIME_PART (encrypted));
-	ciphertext = g_mime_data_wrapper_get_stream (wrapper);
+	ciphertext = g_mime_data_wrapper_get_decoded_stream (wrapper);
 	g_mime_stream_reset (ciphertext);
 	
 	stream = g_mime_stream_mem_new ();
@@ -342,6 +369,7 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCryptoCon
 	/* get the cleartext */
 	if (!(res = g_mime_crypto_context_decrypt (ctx, ciphertext, filtered_stream, err))) {
 		g_object_unref (filtered_stream);
+		g_object_unref (ciphertext);
 		g_object_unref (stream);
 		
 		return NULL;
@@ -349,6 +377,7 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCryptoCon
 	
 	g_mime_stream_flush (filtered_stream);
 	g_object_unref (filtered_stream);
+	g_object_unref (ciphertext);
 	
 	g_mime_stream_reset (stream);
 	parser = g_mime_parser_new ();
@@ -362,7 +391,7 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCryptoCon
 		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
 				     _("Cannot decrypt multipart/encrypted part: failed to parse decrypted content."));
 		
-		g_mime_decryption_result_free (res);
+		g_object_unref (res);
 		
 		return NULL;
 	}
