@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -37,12 +37,18 @@
 #include "gmime-error.h"
 #include "gmime-part.h"
 
+#ifdef ENABLE_DEBUG
+#define d(x) x
+#else
 #define d(x)
+#endif
+
+#define _(x) x
 
 
 /**
  * SECTION: gmime-multipart-signed
- * @title: GMimeMultpartSigned
+ * @title: GMimeMultipartSigned
  * @short_description: Signed MIME multiparts
  * @see_also: #GMimeMultipart
  *
@@ -192,13 +198,13 @@ sign_prepare (GMimeObject *mime_part)
  * g_mime_multipart_signed_sign:
  * @mps: multipart/signed object
  * @content: MIME part to sign
- * @ctx: encryption cipher context
+ * @ctx: encryption crypto context
  * @userid: user id to sign with
- * @hash: preferred digest algorithm
+ * @digest: preferred digest algorithm
  * @err: exception
  *
  * Attempts to sign the @content MIME part with @userid's private key
- * using the @ctx signing context with the @hash algorithm. If
+ * using the @ctx signing context with the @digest algorithm. If
  * successful, the signed #GMimeObject is set as the signed part of
  * the multipart/signed object @mps.
  *
@@ -208,8 +214,8 @@ sign_prepare (GMimeObject *mime_part)
  **/
 int
 g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
-			      GMimeCipherContext *ctx, const char *userid,
-			      GMimeCipherHash hash, GError **err)
+			      GMimeCryptoContext *ctx, const char *userid,
+			      GMimeDigestAlgo digest, GError **err)
 {
 	GMimeStream *stream, *filtered, *sigstream;
 	GMimeContentType *content_type;
@@ -217,14 +223,19 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	GMimePart *signature;
 	GMimeFilter *filter;
 	GMimeParser *parser;
+	const char *protocol;
 	const char *micalg;
 	int rv;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_SIGNED (mps), -1);
-	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), -1);
-	g_return_val_if_fail (ctx->sign_protocol != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), -1);
 	g_return_val_if_fail (GMIME_IS_OBJECT (content), -1);
 	
+	if (!(protocol = g_mime_crypto_context_get_signature_protocol (ctx))) {
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("Signing not supported."));
+		return -1;
+	}
+	
 	/* Prepare all the parts for signing... */
 	sign_prepare (content);
 	
@@ -257,7 +268,7 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	sigstream = g_mime_stream_mem_new ();
 	
 	/* sign the content stream */
-	if ((rv = g_mime_cipher_context_sign (ctx, userid, hash, filtered, sigstream, err)) == -1) {
+	if ((rv = g_mime_crypto_context_sign (ctx, userid, digest, filtered, sigstream, err)) == -1) {
 		g_object_unref (sigstream);
 		g_object_unref (filtered);
 		g_object_unref (stream);
@@ -270,8 +281,8 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	
 	/* set the multipart/signed protocol and micalg */
 	content_type = g_mime_object_get_content_type (GMIME_OBJECT (mps));
-	g_mime_content_type_set_parameter (content_type, "protocol", ctx->sign_protocol);
-	micalg = g_strdup (g_mime_cipher_context_hash_name (ctx, (GMimeCipherHash) rv));
+	g_mime_content_type_set_parameter (content_type, "protocol", protocol);
+	micalg = g_strdup (g_mime_crypto_context_digest_name (ctx, (GMimeDigestAlgo) rv));
 	g_mime_content_type_set_parameter (content_type, "micalg", micalg);
 	g_mime_multipart_set_boundary (GMIME_MULTIPART (mps), NULL);
 	
@@ -282,7 +293,7 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	g_object_unref (parser);
 	
 	/* construct the signature part */
-	content_type = g_mime_content_type_new_from_string (ctx->sign_protocol);
+	content_type = g_mime_content_type_new_from_string (protocol);
 	signature = g_mime_part_new_with_type (content_type->type, content_type->subtype);
 	g_object_unref (content_type);
 	
@@ -293,9 +304,9 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 	g_object_unref (wrapper);
 	
 	/* FIXME: temporary hack, this info should probably be set in
-	 * the CipherContext class - maybe ::sign can take/output a
+	 * the CryptoContext class - maybe ::sign can take/output a
 	 * GMimePart instead. */
-	if (!g_ascii_strcasecmp (ctx->sign_protocol, "application/pkcs7-signature")) {
+	if (!g_ascii_strcasecmp (protocol, "application/pkcs7-signature")) {
 		g_mime_part_set_content_encoding (signature, GMIME_CONTENT_ENCODING_BASE64);
 		g_mime_part_set_filename (signature, "smime.p7m");
 	}
@@ -314,50 +325,60 @@ g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
 /**
  * g_mime_multipart_signed_verify:
  * @mps: multipart/signed object
- * @ctx: encryption cipher context
+ * @ctx: encryption crypto context
  * @err: exception
  *
  * Attempts to verify the signed MIME part contained within the
- * multipart/signed object @mps using the @ctx cipher context.
+ * multipart/signed object @mps using the @ctx crypto context.
  *
- * Returns: a new #GMimeSignatureValidity object on success or %NULL
- * on fail. If the signing fails, an exception will be set on @err to
- * provide information as to why the failure occured.
+ * Returns: a new #GMimeSignatureList object on success or %NULL on fail. If
+ * the verification fails, an exception will be set on @err to provide
+ * information as to why the failure occured.
  **/
-GMimeSignatureValidity *
-g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *ctx,
+GMimeSignatureList *
+g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCryptoContext *ctx,
 				GError **err)
 {
+	const char *supported, *protocol, *micalg;
 	GMimeObject *content, *signature;
-	GMimeDataWrapper *wrapper;
-	GMimeStream *filtered_stream;
-	GMimeFilter *crlf_filter;
 	GMimeStream *stream, *sigstream;
-	const char *protocol, *micalg;
-	GMimeSignatureValidity *valid;
-	GMimeCipherHash hash;
+	GMimeSignatureList *signatures;
+	GMimeStream *filtered_stream;
+	GMimeDataWrapper *wrapper;
+	GMimeFilter *crlf_filter;
+	GMimeDigestAlgo digest;
 	char *content_type;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART_SIGNED (mps), NULL);
-	g_return_val_if_fail (GMIME_IS_CIPHER_CONTEXT (ctx), NULL);
-	g_return_val_if_fail (ctx->sign_protocol != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
 	
 	if (g_mime_multipart_get_count ((GMimeMultipart *) mps) < 2) {
-		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR, "%s",
-			     "Cannot verify multipart/signed part due to missing subparts.");
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
+				     _("Cannot verify multipart/signed part due to missing subparts."));
 		return NULL;
 	}
 	
 	protocol = g_mime_object_get_content_type_parameter (GMIME_OBJECT (mps), "protocol");
 	micalg = g_mime_object_get_content_type_parameter (GMIME_OBJECT (mps), "micalg");
 	
+	supported = g_mime_crypto_context_get_signature_protocol (ctx);
+	
 	if (protocol) {
-		/* make sure the protocol matches the cipher sign protocol */
-		if (g_ascii_strcasecmp (ctx->sign_protocol, protocol) != 0)
+		/* make sure the protocol matches the crypto sign protocol */
+		if (!supported || g_ascii_strcasecmp (supported, protocol) != 0) {
+			g_set_error (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
+				     _("Cannot verify multipart/signed part: unsupported signature protocol '%s'."),
+				     protocol);
 			return NULL;
-	} else {
+		}
+	} else if (supported != NULL) {
 		/* *shrug* - I guess just go on as if they match? */
-		protocol = ctx->sign_protocol;
+		protocol = supported;
+	} else {
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PROTOCOL_ERROR,
+				     _("Cannot verify multipart/signed part: unspecified signature protocol."));
+		
+		return NULL;
 	}
 	
 	signature = g_mime_multipart_get_part (GMIME_MULTIPART (mps), GMIME_MULTIPART_SIGNED_SIGNATURE);
@@ -365,6 +386,8 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	/* make sure the protocol matches the signature content-type */
 	content_type = g_mime_content_type_to_string (signature->content_type);
 	if (g_ascii_strcasecmp (content_type, protocol) != 0) {
+		g_set_error_literal (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
+				     _("Cannot verify multipart/signed part: signature content-type does not match protocol."));
 		g_free (content_type);
 		
 		return NULL;
@@ -404,13 +427,13 @@ g_mime_multipart_signed_verify (GMimeMultipartSigned *mps, GMimeCipherContext *c
 	g_mime_stream_reset (sigstream);
 	
 	/* verify the signature */
-	hash = g_mime_cipher_context_hash_id (ctx, micalg);
-	valid = g_mime_cipher_context_verify (ctx, hash, stream, sigstream, err);
+	digest = g_mime_crypto_context_digest_id (ctx, micalg);
+	signatures = g_mime_crypto_context_verify (ctx, digest, stream, sigstream, err);
 	
 	d(printf ("attempted to verify:\n----- BEGIN SIGNED PART -----\n%.*s----- END SIGNED PART -----\n",
 		  (int) GMIME_STREAM_MEM (stream)->buffer->len, GMIME_STREAM_MEM (stream)->buffer->data));
 	
 	g_object_unref (stream);
 	
-	return valid;
+	return signatures;
 }
