@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -202,6 +202,7 @@ decode_value (const char **in)
 	const char *inptr = *in;
 	
 	decode_lwsp (&inptr);
+	*in = inptr;
 	
 	if (*inptr == '"') {
 		return decode_quoted_string (in);
@@ -209,7 +210,11 @@ decode_value (const char **in)
 		return decode_token (in);
 	}
 	
+#ifndef STRICT_PARSER
+	return decode_token (in);
+#else
 	return NULL;
+#endif
 }
 
 /* This function is basically the same as decode_token()
