@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -34,6 +34,7 @@
 #include <gmime/gmime-data-wrapper.h>
 #include <gmime/gmime-object.h>
 #include <gmime/gmime-part.h>
+#include <gmime/gmime-part-iter.h>
 #include <gmime/gmime-multipart.h>
 #include <gmime/gmime-multipart-encrypted.h>
 #include <gmime/gmime-multipart-signed.h>
@@ -68,7 +69,7 @@
 #include <gmime/gmime-filter-strip.h>
 #include <gmime/gmime-filter-windows.h>
 #include <gmime/gmime-filter-yenc.h>
-#include <gmime/gmime-cipher-context.h>
+#include <gmime/gmime-crypto-context.h>
 #include <gmime/gmime-gpg-context.h>
 
 G_BEGIN_DECLS
