@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,15 +22,10 @@
 #include <config.h>
 #endif
 
+#include <gmime/gmime.h>
+
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <time.h>
-
-#include <gmime/gmime.h>
 
 #include "testsuite.h"
 
@@ -368,13 +363,12 @@ test_iter_remove (void)
 static void
 test_header_sync (void)
 {
-	InternetAddressList *members, *list;
+	InternetAddressList *list;
 	InternetAddress *addr, *ia;
 	GMimeMessage *message;
 	GMimeObject *object;
 	const char *value;
 	GMimePart *part;
-	int i;
 	
 	part = g_mime_part_new_with_type ("application", "octet-stream");
 	object = (GMimeObject *) part;
